package goap

// Interface for every struct that can be part of an ActivityPub object
type BaseApChain interface {
	// Get the next "object" in the chain (or self if last element in chain)
	// Though preferably the last element should always be a BaseObject
	// The 2nd parameter indicates whether the returned value is different from the one the function is called on
	// So true => Is a different object, false => is self
	GetSelfOrBase() (BaseApChain, bool)
	// Convert the chain to a map
	// Should include the rest of the chain (extend the map the underlying object returns)
	MarshalToMap() map[string]any
}

// The minimum data every AP object has
type BaseObject struct {
	Id   string
	Type string
}

func (b *BaseObject) GetSelfOrBase() (BaseApChain, bool) {
	return b, false
}

func (b *BaseObject) MarshalToMap() map[string]any {
	return map[string]any{
		KEY_ID:   b.Id,
		KEY_TYPE: b.Type,
	}
}
